#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "rf01.h"

// send and receive a 16bit-word via spi
uint16_t _rfphy_spi(uint16_t cmd) {
	uint8_t i;
	uint16_t recv = 0;
	// start communication
	RFM12B_SCK_LOW();
	RFM12B_SS_LOW();
	for(i=0; i<16; i++)	{
		if(cmd & 0x8000)
			RFM12B_MOSI_HIGH();
		else
			RFM12B_MOSI_LOW();
		RFM12B_SCK_HIGH();
		recv<<=1;
		if(RFM12B_MISO_HIGH()) {
			recv|=0x0001;
		}
		RFM12B_SCK_LOW();
		cmd<<=1;
	}
	RFM12B_SS_HIGH();
	return recv;
}

void setChannel(uint16_t channel) {
	if(channel>3903)
		channel = 3903;
	_rfphy_spi(0xA000|channel);
}

void setBaud(uint8_t baud) {
	_rfphy_spi(0xC800|baud);
}

void rfphyInitHard() {
	// set default values
	RFM12B_SS_HIGH();
	RFM12B_MOSI_HIGH();
	RFM12B_SCK_LOW();
	// init ports
	RFM12B_SCK_INIT();
	RFM12B_MOSI_INIT();
	RFM12B_MISO_INIT();
	RFM12B_SS_INIT();
}

void rfphyInit(void) {

	rfphyInitHard();
	
	_rfphy_spi(0x8000 | MHZ868 | PF12 | KHZ200 | CRYSTAL);
	_rfphy_spi(0xC200 | CLK166);
	setChannel(RF_CHANNEL_0);
	setBaud(RF_BAUD_19K2);
	_rfphy_spi(0xC600 | AFC | FRQ_REG | DEV15 | FINE_MODE | OFFS_REG | AUTO_ONLY | OFFS_CALC);
	_rfphy_spi(0xC400 | FAST_CLOCK_RECOVERY | DIGITAL_FILTER | 3);
	_rfphy_spi(0xC000 | VDI_CLOCK | LNA14 | RSSI79 | ENABLE_RECEIVER);
	
	/*
	_rfphy_spi(0x8000 | MHZ868 | PF12 | KHZ200 | CRYSTAL);
	setChannel(RF_CHANNEL_0);
	_rfphy_spi(0xC000 | VDI_CLOCK | LNA14 | RSSI79 | ENABLE_RECEIVER);
	_rfphy_spi(0xC200 | CLK166);
	_rfphy_spi(0xC600 | AFC | FRQ_REG | DEVOFF | AUTO_ONLY);
	_rfphy_spi(0xC400 | FAST_CLOCK_RECOVERY | DIGITAL_FILTER | 3);
	setBaud(RF_BAUD_19K2);
	_rfphy_spi(0xDA00 | DISABLE_RESET);
*/
	
	rfphySetRxMode();
}

void rfphySetRxMode() {
	_rfphy_spi(0xCE00 | FIFO_SYNC | FIFOLEVEL(8)); // set FIFO mode
	_rfphy_spi(0xCE00 | FIFO_SYNC | FIFOLEVEL(8) | FIFO_ENABLE | FIFO_FILL); // enable FIFO
	_rfphy_spi(0x0000);
}


void rfphyReceive() {
	uint16_t status = 0;
	uint8_t i;
	RFM12B_SCK_LOW();
	RFM12B_SS_LOW();
	for(i=0; i<16; i++)	{
		RFM12B_SCK_HIGH();
		status<<=1;
		if(RFM12B_MISO_HIGH()) {
			status|=0x0001;
		}
		RFM12B_SCK_LOW();
		 asm volatile ("nop");
	}
	
	if( (status&RF_STATUS_RSSI)) {
		uartWordLn(status);
	}
	// Data available?
	if( status&RF_STATUS_FFIT ) {
		uint8_t c = 0;
		for(i=0; i<16; i++)	{
			RFM12B_SCK_HIGH();
			c<<=1;
			if(RFM12B_MISO_HIGH()) {
				c|=0x0001;
			}
			RFM12B_SCK_LOW();
			asm volatile ("nop");
		}
		RFM12B_SS_HIGH();

		uartByteLn(c);
		if( !(status&RF_STATUS_RSSI)) {
			rfphySetRxMode();
		}
	}
	else {
		RFM12B_SS_HIGH();
	}
}
